/*
 * libzdt - zdt's own C Library.
 * Copyright (C) 1999,2000 Jeremy Tregunna. All rights reserved.
 *
 * libzdt is distributed under a modified version of the BSD license.
 * You should have received a copy of this license in the Documentation
 * directory.
 */
#include <zdt/Types.h>
#include <zdt/rc5.h>

ulint P = 0xb7e15163;
ulint Q = 0x9e3779b9;

#define ROTL(x,y) (((x) << (y & (rc5_w - 1))) | ((x) >> (rc5_w - (y & (rc5_w - 1)))))
#define ROTR(x,y) (((x) >> (y & (rc5_w - 1))) | ((x) << (rc5_w - (y & (rc5_w - 1)))))

void
_zcrypt_rc5_encrypt(ulint *pt,ulint *ct,ulint *SU)
{
	ulint i;
	ulint A = pt[0] + SU[0];
	ulint B = pt[1] + SU[1];

	for(i = 1; i < rc5_r; i++) {
		A = ROTL(A^B,B) + SU[2 * i];
		B = ROTL(B^A,A) + SU[2 * i + 1];
	}

	ct[0] = A;
	ct[1] = B;
}

void
_zcrypt_rc5_decrypt(ulint *ct,ulint *pt,ulint *SU)
{
	ulint i;
	ulint B = ct[1];
	ulint A = ct[0];

	for(i = rc5_r; i > 0; i--) {
		B = ROTR(B - SU[2 * i + 1],A) ^ A;
		A = ROTR(A - SU[2 * i],B) ^ B;
	}

	pt[1] = B - SU[1];
	pt[0] = A - SU[0];
}

void
_zcrypt_rc5_setup(uint8 *K,ulint *SU)
{
	ulint i,j,k;
	ulint u = rc5_w / 8;
	ulint A,B;
	ulint L[rc5_c];

	for(i = rc5_b - 1,L[rc5_c - 1] = 0; i != -1; i--) {
		L[i / u] = (L[i / u] << 8) + K[i];
	}

	for(SU[0] = P,i = 1; i < rc5_t; i++) {
		SU[i] = SU[i - 1] + Q;
	}

	for(A = B = i = j = k = 0; k < 3 * rc5_t; k++,i = (i + 1) % rc5_t,j = (j + 1) % rc5_c) {
		A = SU[i] = ROTL(SU[i] + (A + B),3);
		B = L[j] = ROTL(L[j] + (A + B),(A + B));
	}
}
